@@ -14,6 +14,7 @@ setClass("dwi",
                         voxelext = "numeric",
                         level  = "numeric",
                         orientation = "integer",
+                        rotation = "matrix",
                         source = "character"),
          )
 
@@ -210,11 +211,12 @@ setClass("dwiFiber",
         )
 setClass("dwiMixtensor",
          representation(method = "character",
+                        model  = "character",
                         ev     = "array",#length 2 (eigenvalues)
                         mix    = "array",
                         orient = "array",
                         order  = "array",
-                        p      = "narray", # p in "method"=="Jian"
+                        p      = "numeric", # p in "method"=="Jian"
                         th0    = "array",
                         sigma  = "array",
                         scorr  = "array",
@@ -243,7 +245,7 @@ setClass("dwiMixtensor",
             return(invisible(FALSE))
           }
           if (any(dim(object@order)!=object@ddim)) {
-            cat("  \n")
+            cat("invalid dimension of order array orient \n")
             return(invisible(FALSE))
           }
           if (any(object@mix<0)) {
